import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import time

class Stats:
    __slots__ = (
        "volume",
        "count",
        "profit_count",
        "loss_count",
        "win_rate",
        "profit_value",
        "loss_value",
        "net_pnl",  # 价差净收益
        "base_funding_fee",
        "hedge_funding_fee",
        "funding_pnl",
        "leg_count",
        "leg_volume",
        "leg_pnl",
        "total_pnl",  # net_pnl + funding_pnl + leg_pnl
        "fee",
        "rebate",
    )

    def __init__(self):
        self.volume = 0.0
        self.count = 0
        self.profit_count = 0
        self.loss_count = 0
        self.win_rate = 0.0
        self.profit_value = 0.0
        self.loss_value = 0.0
        self.net_pnl = 0.0
        self.base_funding_fee = 0.0
        self.hedge_funding_fee = 0.0
        self.funding_pnl = 0.0
        self.leg_count = 0
        self.leg_volume = 0.0
        self.leg_pnl = 0.0
        self.total_pnl = 0.0
        self.fee = 0.0
        self.rebate = 0.0


class Statistics:
    def __init__(self, symbols=['BTC_USDT'], balance=10000, fee_rate=0.0006, exchange_len=3):
        self.now_balance = balance
        self.max_balance = balance
        self.today_start_balance = balance

        self.start_balance = balance
        self.start_time = int(time.time() * 1000)

        self.fee_rate = fee_rate

        self.symbol_stats = {symbol: Stats() for symbol in symbols}

        self.total_stats = Stats()

        if exchange_len:
            self.exchange_info = {account_id: {"init_balance":0, "max_balance":0, "now_balance":0, "available_balance":0, "total_pnl":0} for account_id in range(exchange_len)}
        

    def update_today_balance(self, balance):
        self.today_start_balance = balance

    def update_exchange_info(self, account_id, balance):
        if account_id not in self.exchange_info:
            self.exchange_info[account_id] = {"init_balance":0, "max_balance":0, "now_balance":0, "available_balance":0, "total_pnl":0}
        if self.exchange_info[account_id]["init_balance"] == 0:
            self.exchange_info[account_id]["init_balance"] = balance["balance"]
        self.exchange_info[account_id]["now_balance"] = balance["balance"]
        self.exchange_info[account_id]["available_balance"] = balance["available_balance"]
        if balance["balance"] >= self.exchange_info[account_id]["max_balance"]:
            self.exchange_info[account_id]["max_balance"] = balance["balance"]
        self.exchange_info[account_id]["total_pnl"] = self.exchange_info[account_id]["now_balance"] - self.exchange_info[account_id]["init_balance"]

    def update_balance(self, now_balance):
        self.now_balance = now_balance
        if self.max_balance < now_balance:
            self.max_balance = now_balance

    



class Order:
    __slots__ = (
        "account_id",
        "symbol",
        "side",
        "pos_side",
        "price",
        "amount",
        "status",
        "is_market",
        "filled_avg_price",
        "filled_qty",
        "reduce_only",
        "fee",
        "send_time",
        "ack_time",
        "open_price",
        "is_single_leg",
        "is_twap",
    )

    def __init__(
        self,
        account_id,
        symbol,
        side,
        pos_side,
        price,
        amount,
        send_time=0,
        reduce_only=False,
        is_market=False,
        status="New",
        filled_avg_price=0.0,
        filled_qty=0.0,
        fee=0.0,
        ack_time=0,
        open_price=0.0,
        is_single_leg=False,
        is_twap=False,
    ):
        self.account_id = account_id
        self.symbol = symbol
        self.side = side
        self.pos_side = pos_side
        self.price = price
        self.amount = amount
        self.status: str = status
        self.is_market = is_market
        self.filled_avg_price = filled_avg_price
        self.filled_qty = filled_qty
        self.reduce_only = reduce_only
        self.fee: float = fee
        self.send_time = send_time
        self.ack_time: int = ack_time
        self.open_price = open_price
        self.is_single_leg = is_single_leg
        self.is_twap = is_twap

    def update(self, order, entry_price):
        self.open_price = entry_price
        self.status = order["status"]
        self.filled_avg_price = order["filled_avg_price"]
        self.filled_qty = order["filled"]
        self.ack_time = order["timestamp"]
        self.pos_side = order["pos_side"]
        self.side = order["side"]

    def to_dict(self):
        """返回包含所有属性的字典"""
        return {
            "account_id": self.account_id,
            "symbol": self.symbol,
            "side": self.side,
            "pos_side": self.pos_side,
            "price": self.price,
            "amount": self.amount,
            "status": self.status,
            "is_market": self.is_market,
            "filled_avg_price": self.filled_avg_price,
            "filled_qty": self.filled_qty,
            "reduce_only": self.reduce_only,
            "fee": self.fee,
            "send_time": self.send_time,
            "ack_time": self.ack_time,
            "open_price": self.open_price,
            "is_single_leg": self.is_single_leg,
            "is_twap": self.is_twap,
        }
    
    @classmethod
    def from_dict(cls, data):
        """从字典恢复 Order 实例"""
        order = cls(
            account_id=data["account_id"],
            symbol=data["symbol"],
            side=data["side"],
            pos_side=data["pos_side"],
            price=data["price"],
            amount=data["amount"],
            send_time=data["send_time"],
            reduce_only=data["reduce_only"],
            is_market=data["is_market"]
        )
        order.status = data["status"]
        order.filled_avg_price = data["filled_avg_price"]
        order.filled_qty = data["filled_qty"]
        order.fee = data["fee"]
        order.ack_time = data["ack_time"]
        order.open_price = data["open_price"]
        order.is_single_leg = data.get("is_single_leg", False)
        order.is_twap = data.get("is_twap", False)
        return order


class OrderManager:
    def __init__(self):
        # current_order 存储挂单的 symbol 和 order_id
        # local_orders 存储挂单的 order_id 和 local_order
        self.local_orders: dict[str, Order] = {}
        self.current_order: dict[str, str] = {}

    def update_current_order(self, symbol: str, order_id: str):
        self.current_order[symbol] = order_id

    def remove_current_order(self, symbol: str):
        if symbol in self.current_order:
            del self.current_order[symbol]

    def get_current_order_id(self, symbol: str) -> str:
        return self.current_order.get(symbol, None)

    def add_order(self, order_id: str, order: Order):
        self.local_orders[order_id] = order

    def update_order(self, order_id: str, update_data: dict, entry_price: float):
        order = self.local_orders.get(order_id)
        if order:
            order.update(update_data, entry_price)

    def remove_order(self, order_id: str):
        self.local_orders.pop(order_id, None)

    def get_order(self, order_id: str) -> Order:
        return self.local_orders.get(order_id)

    def clear_finished_orders(self):
        to_delete = [oid for oid, o in self.local_orders.items() if o.status in ("FILLED", "CANCELED")]
        for oid in to_delete:
            del self.local_orders[oid]

    def to_dict(self):
        """将 OrderManager 状态转换为 JSON 兼容的字典"""
        return {
            "local_orders": {order_id: order.to_dict() for order_id, order in self.local_orders.items()}
        }

    @classmethod
    def from_dict(cls, data):
        """从字典恢复 OrderManager 实例"""
        manager = cls()
        if "local_orders" in data:
            for order_id, order_data in data["local_orders"].items():
                order = Order.from_dict(order_data)
                manager.add_order(order_id, order)
        return manager





import json
import uuid
import sys
import os
sys.path.append(os.path.dirname(os.path.abspath(__file__)) + '/../')

from datetime import datetime
from typing import Any, Dict, List, Optional, Callable

from enums.risk_signal_types import RiskScope, RiskAction


class RiskSignal:
    """
    风控信号容器。
    - 支持按层级记录风控事件：tick / order / global / symbol
    - 每个事件包含：action, message, timestamp, target_id, context
    - 空对象代表"无风控问题"
    """

    def __init__(self) -> None:
        self._signals: Dict[RiskScope, List[Dict[str, Any]]] = {
            RiskScope.TICK: [],
            RiskScope.ORDER: [],
            RiskScope.GLOBAL: [],
            RiskScope.SYMBOL: [],
        }

    def add(
        self,
        scope: RiskScope,
        action: RiskAction = RiskAction.NONE,
        message: str = "",
        target_id: Optional[str] = None,
        context: Optional[Dict[str, Any]] = None,
    ) -> str:
        """
        新增风控事件，返回事件唯一 ID。
        """
        event_id = uuid.uuid4().hex
        event = {
            "id": event_id,
            "action": action,
            "message": message,
            "timestamp": datetime.utcnow().isoformat() + "Z",
            "target_id": target_id,
            "context": context or {},
        }
        self._signals[scope].append(event)
        return event_id

    def remove_event(self, scope: RiskScope, event_id: str) -> bool:
        """
        根据事件 ID 删除指定层级的事件。返回是否删除成功。
        """
        events = self._signals.get(scope, [])
        original_len = len(events)
        self._signals[scope] = [e for e in events if e.get("id") != event_id]
        return len(self._signals[scope]) != original_len

    def resolve(
        self,
        scope: Optional[RiskScope] = None,
        target_id: Optional[str] = None,
        predicate: Optional[Callable[[Dict[str, Any]], bool]] = None,
    ) -> int:
        """
        通过条件删除事件，返回删除数量。
        - scope 为 None 时表示所有层级
        - target_id 匹配事件 target_id
        - predicate 为自定义匹配函数，返回 True 表示删除
        """
        scopes = [scope] if scope is not None else list(self._signals.keys())
        removed = 0
        for sc in scopes:
            kept: List[Dict[str, Any]] = []
            for e in self._signals[sc]:
                match_target = (target_id is None or e.get("target_id") == target_id)
                match_pred = (predicate(e) if predicate is not None else True)
                if match_target and match_pred:
                    removed += 1
                else:
                    kept.append(e)
            self._signals[sc] = kept
        return removed

    def clear_scope(self, scope: RiskScope) -> None:
        """
        清空某个层级的所有事件。
        """
        self._signals[scope] = []

    def clear_all(self) -> None:
        """
        清空所有层级事件。
        """
        for sc in self._signals.keys():
            self._signals[sc] = []

    def is_empty(self) -> bool:
        return all(len(v) == 0 for v in self._signals.values())

    def __bool__(self) -> bool:
        return not self.is_empty()

    def latest_per_scope(self) -> Dict[RiskScope, Optional[Dict[str, Any]]]:
        out: Dict[RiskScope, Optional[Dict[str, Any]]] = {}
        for scope, events in self._signals.items():
            out[scope] = events[-1] if events else None
        return out

    def to_dict(self) -> Dict[str, Any]:
        def ser_event(e: Dict[str, Any]) -> Dict[str, Any]:
            return {
                "id": e["id"],
                "action": e["action"].name,
                "message": e["message"],
                "timestamp": e["timestamp"],
                "target_id": e["target_id"],
                "context": e["context"],
            }

        return {
            "signals": {
                scope.name.lower(): [ser_event(e) for e in events]
                for scope, events in self._signals.items()
            },
        }

    @classmethod
    def ok(cls) -> "RiskSignal":
        return cls()


if __name__ == "__main__":
    # TODO: 这个地方的问题在于，生成了 eid，但是 risk_mananger.py 中不好管理。
    # 需要设计一个方法，用于管理 eid，并且能够将 eid 和对应的风控事件对应，当对应的风控事件发生并解除的时候，能够通过 eid 删除对应的事件。

    # 使用示例：创建 RiskSignal，添加/删除多个事件并输出摘要
    rs = RiskSignal()

    # 添加 TICK 层多个事件
    eid1 = rs.add(
        scope=RiskScope.TICK,
        action=RiskAction.STOP_OPEN,
        message="价差异常，暂停开仓",
        target_id="BTC_USDT",
        context={"spread": 0.005},
    )
    eid2 = rs.add(
        scope=RiskScope.TICK,
        action=RiskAction.STOP_ALL,
        message="延迟过高，停止所有交易",
        target_id="BTC_USDT",
        context={"delay": 1000},
    )

    # 添加 ORDER 层事件
    eid3 = rs.add(
        scope=RiskScope.ORDER,
        action=RiskAction.STOP_ALL,
        message="订单发送失败，停止所有交易",
        target_id="BTC_USDT",
        context={"err": "EXCHANGE_ERROR"},
    )

    # 添加 global 层 OK 事件
    eid4 = rs.add(
        scope=RiskScope.GLOBAL,
        action=RiskAction.STOP_ALL,
        message="账户杠杆过高，停止开仓",
        target_id="ALL",
        context={"leverage": 8},
    )

    print("是否为空信号:", rs.is_empty())

    # 按层级查看最新事件
    latest = rs.latest_per_scope()
    print("各层级最新事件:")
    for scope, event in latest.items():
        if event is None:
            print(f"  {scope.name.lower()}: None")
        else:
            print(
                f"  {scope.name.lower()}: "
                f"action={event['action'].name}, "
                f"message={event['message']}, target_id={event['target_id']}"
            )

    # 序列化输出
    print("序列化字典:")
    print(json.dumps(rs.to_dict(), ensure_ascii=False, indent=2))

    # 按 ID 删除一个 symbol 事件
    print("按 ID 删除一个 symbol 事件:", eid1)
    rs.remove_event(RiskScope.TICK, eid1)
    print(json.dumps(rs.to_dict(), ensure_ascii=False, indent=2))

    # 按条件删除剩余的 symbol 事件（例如同一 target_id）
    print("删除 target_id=BTC_USDT 的剩余 symbol 事件")
    removed = rs.resolve(scope=RiskScope.TICK, target_id="BTC_USDT")
    print("删除数量:", removed)
    print(json.dumps(rs.to_dict(), ensure_ascii=False, indent=2))

    # 清空 account 层
    print("清空 ORDER 层")
    rs.clear_scope(RiskScope.ORDER)
    print(json.dumps(rs.to_dict(), ensure_ascii=False, indent=2))

    # 清空所有层级
    print("清空所有层级")
    rs.clear_all()
    print(json.dumps(rs.to_dict(), ensure_ascii=False, indent=2))




import sys
import os
sys.path.append(os.path.dirname(os.path.abspath(__file__)) + '/../')

from typing import Any, Dict
from risk_manager import RiskManager


def simulate_receipt_failures() -> None:
    rm = RiskManager()
    # 映射 index->exchange 名称，便于测试输出
    rm.ex_map = {0: "BinanceSwap", 1: "OkxSwap", "BinanceSwap": 0, "OkxSwap": 1}

    base = 1_000_000_000_000  # 虚拟时间基准（ms）
    now = base
    rm._now_ms = lambda: now  # 覆盖时间函数，便于场景控制

    symbol = "BTC_USDT"
    err_receipt: Dict[str, Any] = {"Err": {"code": 2000, "error": "mock"}}

    print("场景：10秒内连续5次失败, 触发 RiskSignal")
    for i in range(5):
        rs = rm.check_order_receipt_risk(0, symbol, err_receipt)
        print(f"第{i+1}次, signal={rs.to_dict() if rs else {}}")
        now += 1000  # 每次+1s，10秒内累计

    print("场景：超出10秒窗口但不足5次, 不触发")
    now = base
    for i in range(4):
        rs = rm.check_order_receipt_risk(0, symbol, err_receipt)
        print(f"第{i+1}次, signal={rs.to_dict() if rs else {}}")
        now += 4000  # 每次+4s，超过窗口累计不足5次


if __name__ == "__main__":
    simulate_receipt_failures()





import unittest
import sys
import os
sys.path.append(os.path.dirname(os.path.abspath(__file__)) + '/../')

from typing import Any, Dict

from risk_manager import RiskManager


class TestOrderReceiptRisk(unittest.TestCase):
    def setUp(self):
        self.rm = RiskManager()
        self.rm.ex_map = {0: "BinanceSwap", 1: "OkxSwap", "BinanceSwap": 0, "OkxSwap": 1}
        self.base = 1_000_000_000_000
        self.now = self.base
        self.rm._now_ms = lambda: self.now
        self.symbol = "BTC_USDT"
        self.err_receipt: Dict[str, Any] = {"Err": {"code": 2000, "error": "mock"}}

    def test_trigger_after_5_within_10s(self):
        # 前4次不触发，第5次触发
        for i in range(4):
            rs = self.rm.check_order_receipt_risk(0, self.symbol, self.err_receipt)
            self.assertTrue(rs.is_empty())
            self.now += 1000
        rs = self.rm.check_order_receipt_risk(0, self.symbol, self.err_receipt)
        self.assertFalse(rs.is_empty())
        out = rs.to_dict()
        self.assertIn("signals", out)
        self.assertTrue(out["signals"]["order"])

    def test_expire_if_not_enough_hits(self):
        # 4次，每次+4s，超过10s窗口累计不足5次， 超过窗口时间且未达阈值，销毁旧窗口，重新从1开始，不触发
        for i in range(4):
            rs = self.rm.check_order_receipt_risk(0, self.symbol, self.err_receipt)
            self.assertTrue(rs.is_empty())
            self.now += 4000
        # 窗口应当已过期，再次开始累计，从1开始，不触发
        rs = self.rm.check_order_receipt_risk(0, self.symbol, self.err_receipt)
        self.assertTrue(rs.is_empty())


if __name__ == "__main__":
    unittest.main()





import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from typing import List, Dict, Any
from utils.position_manager import HedgePositionManager


def symbol_exposure_check(
    symbols: List[str],
    position_manager: HedgePositionManager,
    abnormal_position_exposure_threshold: float = 0.05,
) -> bool:
    """
    检查 symbol 的敞口是否正常
    如果检查 symbol 的仓位发生了较大敞口，则应该将 symbol 加到 result 中返回
    """
    result = {'exposure': list()}
    if not symbols or len(symbols) <= 0:
        return result
    
    for symbol in symbols:
        pos_0, pos_1 = position_manager.get_hedge_positions(symbol)
        pos_0_size = pos_0.amount
        pos_1_size = pos_1.amount

        pos_size_delta = abs(pos_0_size - pos_1_size)

        if pos_1_size != 0 and pos_size_delta / pos_1_size > abnormal_position_exposure_threshold:
            result['exposure'].append(symbol)
        elif (pos_1_size == 0 and pos_0_size != 0) or (pos_1_size != 0 and pos_0_size == 0):
            result['exposure'].append(symbol)
        
    return result

def symbol_notional_exposure_check(
    symbols: List[str],
    bbo: List[Dict[str, Any]],
    position_manager: HedgePositionManager,
    abnormal_position_exposure_threshold: float = 0.05,
) -> bool:
    """
    检查 symbol 的名义市值敞口是否正常
    如果检查 symbol 的仓位发生了较大敞口，则应该将 symbol 加到 result 中返回
    """
    result = {'exposure': list()}
    if not symbols or not position_manager:
        return result
    
    for symbol in symbols:
        if symbol not in bbo[0] or symbol not in bbo[1]:
            continue
        pos_0, pos_1 = position_manager.get_hedge_positions(symbol)
        taker_notional = pos_0.amount * (bbo[0][symbol][2] + bbo[0][symbol][0]) * 0.5
        maker_notional = pos_1.amount * (bbo[1][symbol][2] + bbo[1][symbol][0]) * 0.5

        if (maker_notional != 0 and taker_notional / maker_notional - 1 > abnormal_position_exposure_threshold) \
            or (taker_notional != 0 and maker_notional / taker_notional - 1 > abnormal_position_exposure_threshold):
            result['exposure'].append(symbol)
        elif (maker_notional == 0 and taker_notional != 0) or (taker_notional == 0 and maker_notional != 0):
            result['exposure'].append(symbol)
    
    return result
    

def symbol_single_leg_check(symbols: List[str], position_manager: HedgePositionManager) -> bool:
    """
    检查 symbol 的单腿是否正常
    如果检查 symbol 发生了单腿，则应该将该 symbol 加到 result 中返回
    """
    result = {'single_leg': list()}
    if not symbols or len(symbols) <= 0:
        return result

    for symbol in symbols:
        pos_0, pos_1 = position_manager.get_hedge_positions(symbol)
        pos_0_size = pos_0.amount
        pos_1_size = pos_1.amount

        if (pos_0_size != 0 and pos_1_size == 0) or (pos_0_size == 0 and pos_1_size != 0):
            result['single_leg'].append(symbol)
    
    return result


import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from utils.order_manager import Order
from typing import Tuple

class Position:
    def __init__(self):
        self.amount = 10
        self.avg_price = 0
        self.side = ''
        self.hedge_index = 0
        self.update_time = 0

    def __repr__(self):
        return f"Position(amount={self.amount}, avg_price={self.avg_price}, side={self.side}, hedge_index={self.hedge_index})"

class HedgePositionManager:
    def __init__(self, symbols: list[str] = ['BTC_USDT'], base_index: int = 2):
        print(f"Mock HedgePositionManager class 初始化")

        self.symbols = symbols
        self.base_index = 2
        self.positions = [{symbol: Position() for symbol in self.symbols} for _ in range(self.base_index)]

    def update_positions(self, index, positions, init_pos=False):
        """
        更新交易所 index 的持仓列表。
        - init_pos=True: 直接按原逻辑更新（用于初始化），不做连续误差判断。
        - init_pos=False: 当 pos['amount'] 与本地值不相等时，累计误差次数；仅当连续2次不相等时才进行更新；
                          如果上次不相等这次相等，则将误差次数归 0。
        """
        for pos in positions:
            symbol = pos["symbol"]
            # 方案B：过滤未订阅且 amount 为 0 的 symbol（跳过无效条目）
            if symbol not in self.symbols:
                continue

            # 确保当前 index 的 symbol 初始化
            if symbol not in self.positions[index]:
                # 初始化未知 symbol
                self.positions[index][symbol] = Position()
            if symbol not in self.amount_mismatch_counts[index]:
                self.amount_mismatch_counts[index][symbol] = 0

            local_pos: LocalPosition = self.positions[index][symbol]

            if init_pos:
                # 原始更新逻辑（不启用误差门控）
                local_pos.amount = pos["amount"]
                if pos["amount"] != 0:
                    local_pos.avg_price = pos["entry_price"]
                    local_pos.side = pos["side"]
                local_pos.hedge_index = 0
                if index:
                    self.positions[0][symbol].hedge_index = index
                if "timestamp" in pos:
                    local_pos.update_time = pos["timestamp"]
                elif 'utime' in pos:
                    local_pos.update_time = pos['utime']
                local_pos.unrealized_pnl = pos['unrealized_pnl']
                local_pos.return_ratio = local_pos.unrealized_pnl / (local_pos.avg_price * local_pos.amount) if local_pos.amount != 0 and local_pos.avg_price != 0 else 0.0
                self.positions[index][symbol] = local_pos
                # 初始化路径下重置计数
                self.amount_mismatch_counts[index][symbol] = 0
                continue

            # 误差门控逻辑
            incoming_amount = pos["amount"]
            current_amount = local_pos.amount
            incoming_entry_price = pos["entry_price"]
            current_entry_price = local_pos.avg_price
            mismatch = (incoming_amount != current_amount) or (incoming_entry_price != current_entry_price)

            if mismatch:
                # 累计误差
                self.amount_mismatch_counts[index][symbol] = self.amount_mismatch_counts[index].get(symbol, 0) + 1
                if self.amount_mismatch_counts[index][symbol] >= 2:
                    # 连续两次不相等，执行更新并清零计数
                    local_pos.amount = incoming_amount
                    if incoming_amount != 0:
                        local_pos.avg_price = pos["entry_price"]
                        local_pos.side = pos["side"]
                    # 其他字段仍然更新
                    local_pos.hedge_index = 0
                    if index:
                        self.positions[0][symbol].hedge_index = index
                    if "timestamp" in pos:
                        local_pos.update_time = pos["timestamp"]
                    elif 'utime' in pos:
                        local_pos.update_time = pos['utime']
                    local_pos.unrealized_pnl = pos['unrealized_pnl']
                    local_pos.return_ratio = local_pos.unrealized_pnl / (local_pos.avg_price * local_pos.amount) if local_pos.amount != 0 and local_pos.avg_price != 0 else 0.0
                    self.positions[index][symbol] = local_pos
                    # 清零计数
                    self.amount_mismatch_counts[index][symbol] = 0
                else:
                    # 未达到阈值，仅计数，不更新
                    continue
            else:
                # 相等时，如之前有误差计数则归 0，并继续按正常逻辑更新非金额字段
                if self.amount_mismatch_counts[index].get(symbol, 0) != 0:
                    self.amount_mismatch_counts[index][symbol] = 0
                # 相等情况下，更新除 amount/avg_price/side 的其余字段（保持原有规则）
                if "timestamp" in pos:
                    local_pos.update_time = pos["timestamp"]
                elif 'utime' in pos:
                    local_pos.update_time = pos['utime']
                local_pos.unrealized_pnl = pos['unrealized_pnl']
                local_pos.return_ratio = local_pos.unrealized_pnl / (local_pos.avg_price * local_pos.amount) if local_pos.amount != 0 and local_pos.avg_price != 0 else 0.0
                local_pos.hedge_index = 0
                if index:
                    self.positions[0][symbol].hedge_index = index
                self.positions[index][symbol] = local_pos

    def update_position_by_order(self, index, local_order: Order, trader=None):
        # 改进时间戳检查逻辑，确保持仓成本价更新不会失败
        local_position = self.positions[index][local_order.symbol]
        
        # 如果订单时间戳比持仓更新时间早，但有成交数据，仍然允许更新
        # 这是因为订单回调可能比持仓回调先到达
        if local_position.update_time >= local_order.ack_time and local_order.filled_qty == 0:
            # 只有当订单没有成交且时间戳更早时才跳过更新
            return 0.0

        # 记录更新前的状态
        old_amount = local_position.amount
        old_avg_price = local_position.avg_price
        old_side = local_position.side

        if not local_position.side:
            # 第一笔开仓单
            local_position.amount = local_order.filled_qty
            local_position.avg_price = local_order.filled_avg_price
            # 避免用 None 覆盖已有值
            if local_order.pos_side is not None:
                local_position.side = local_order.pos_side
            local_position.update_time = local_order.ack_time
            self.positions[index][local_order.symbol] = local_position
            
            # 打印开仓日志
            log_msg = f"[持仓更新] 开仓 - 交易所{index} {local_order.symbol}: 数量={local_order.filled_qty:.6f}, 成本价={local_order.filled_avg_price:.6f}, 方向={local_order.pos_side}"
            trader.log(f"index: {index}, 更新仓位:[{index}], 订单方向:{local_order.pos_side}, 仓位方向:{local_position.side}", level="INFO", web=False)
            if trader:
                trader.log(log_msg, level="INFO", color="blue", web=False)
            else:
                print(log_msg)
            
            return 0.0
            
        elif local_order.reduce_only:
            # 平仓单
            if trader:
                trader.log(
                    f"[持仓更新][平仓前] 交易所{index} {local_order.symbol}: old_amount={old_amount:.6f}, filled_qty={local_order.filled_qty:.6f}, side={old_side}",
                    level="INFO", color="blue", web=False
                )
            # 扣减数量
            new_amount = max(0.0, old_amount - local_order.filled_qty)
            local_position.amount = new_amount
            local_position.update_time = local_order.ack_time
            self.positions[index][local_order.symbol] = local_position
            
            # 计算平仓价差PnL
            if old_avg_price > 0:
                if old_side == "Long":
                    pnl = (local_order.filled_avg_price - old_avg_price) * local_order.filled_qty
                else:  # Short
                    pnl = (old_avg_price - local_order.filled_avg_price) * local_order.filled_qty
            else:
                pnl = 0
                
            # 打印平仓日志
            log_msg = f"[持仓更新] 平仓 - 交易所{index} {local_order.symbol}: 平仓数量={local_order.filled_qty:.6f}, 平仓价={local_order.filled_avg_price:.6f}, 原成本价={old_avg_price:.6f}, 平仓价差PnL={pnl:.6f}, 剩余数量={local_position.amount:.6f}"
            if trader:
                trader.log(log_msg, level="INFO", color="blue", web=False)
            else:
                trader.log(log_msg, level="INFO", web=False)
            
            if local_position.amount == 0:
                local_position.avg_price = 0
                local_position.side = None
                zero_msg = f"[持仓更新] 清仓 - 交易所{index} {local_order.symbol}: 持仓已清零"
                if trader:
                    trader.log(zero_msg, level="INFO", color="blue", web=False)
                else:
                    trader.log(zero_msg, level="INFO", web=False)
            if trader:
                trader.log(
                    f"[持仓更新][平仓后] 交易所{index} {local_order.symbol}: new_amount={local_position.amount:.6f}, side={local_position.side}",
                    level="INFO", color="blue", web=False
                )
            
            return pnl
                
        elif not local_order.reduce_only:
            # 补仓单
            if trader:
                trader.log(
                    f"[持仓更新][加仓前] 交易所{index} {local_order.symbol}: old_amount={old_amount:.6f}, old_avg={old_avg_price:.6f}, filled_qty={local_order.filled_qty:.6f}, filled_avg={local_order.filled_avg_price:.6f}, side={old_side}",
                    level="INFO", color="blue", web=False
                )
            # 计算加仓后的加权成本与数量
            new_amount = old_amount + local_order.filled_qty
            if new_amount > 0:
                new_avg_price = (
                    old_amount * old_avg_price + local_order.filled_qty * local_order.filled_avg_price
                ) / new_amount
            else:
                new_avg_price = 0.0
            local_position.avg_price = new_avg_price
            local_position.amount = new_amount
            # 避免用 None 覆盖已有值
            if local_order.pos_side is not None:
                local_position.side = local_order.pos_side
            local_position.update_time = local_order.ack_time
            self.positions[index][local_order.symbol] = local_position
            
            # 打印补仓日志
            log_msg = (
                f"[持仓更新] 补仓 - 交易所{index} {local_order.symbol}: 补仓数量={local_order.filled_qty:.6f}, 补仓价={local_order.filled_avg_price:.6f}, "
                f"原成本价={old_avg_price:.6f}, 新成本价={new_avg_price:.6f}, 总数量={local_position.amount:.6f}"
            )
            if trader:
                trader.log(log_msg, level="INFO", color="blue", web=False)
                trader.log(
                    f"[持仓更新][加仓后] 交易所{index} {local_order.symbol}: new_amount={local_position.amount:.6f}, new_avg={local_position.avg_price:.6f}, side={local_position.side}",
                    level="INFO", color="blue", web=False
                )
            else:
                print(log_msg)
            
            return 0.0

    def get_positions(self, index, symbol):
        return self.positions[index].get(symbol, Position())

    def get_hedge_positions(self, symbol: str) -> Tuple[Position, Position]:
        base_pos = self.positions[0][symbol]
        hedge_pos = self.positions[1][symbol]
        return [base_pos, hedge_pos]




from utils.trader import Trader
from utils.statistics import Statistics
from utils.position_manager import HedgePositionManager
from utils.signal_manager import SignalManager
from risk_manager import risk_managed                      # 导入库


##############################################################################
# 以下是 Mock Strategy 的类，用于模拟真实交易场景中的策略使用。
##############################################################################
@risk_managed()               # 使用装饰器
class Strategy:
    
    def __init__(self, cex_configs={}, dex_configs={}, config=None, trader=Trader()):
        self.has_account = True
        self.base_index = 2
        self.config = config
        self.trader = trader

        self.local_symbol_order = {}
        self.force_stop = False  # 强制停止标志，用于最大回撤时的强制停止
        self.total_account_stat = Statistics()

        self.instruments = [{} for _ in range(self.base_index)]
        self.ex_map = {'BinanceSwap': 0, 'OkxSwap': 1, 0: 'BinanceSwap', 1: 'OkxSwap'}
        self.bbo = [{} for _ in range(self.base_index)]
        self.balances = [{} for _ in range(self.base_index)]


    def prepare_symbols(self):
        self.symbols = [["BTC_USDT"], ["BTC_USDT"]]

        self.fundings = [
            {symbol: {"funding_rate": 0, "funding_interval": 8,
                        "next_funding_at": 0} for symbol in self.symbols[idx]}
            for idx in range(self.base_index)
        ]

        self.pm = HedgePositionManager()
        self.signal_manager = SignalManager()

    def start(self):
        print(f"这是策略启动的逻辑")

    def subscribes(self):
        print(f"这是订阅逻辑")
    
    def on_bbo(self, exchange, bbo):
        risk_signal = self.risk_manager.check_tick_risk()                 # 检查 tick 级别风控
        if risk_signal and not risk_signal.is_empty():
            print(f"bbo 风控告警: {risk_signal.to_dict()}")
            print(f"---------------------------------------------------------")

    def on_depth(self, exchange, depth):
        risk_signal = self.risk_manager.check_tick_risk()                 # 检查 tick 级别风控
        if risk_signal and not risk_signal.is_empty():
            print(f"depth 风控告警: {risk_signal.to_dict()}")
            print(f"---------------------------------------------------------")

    def on_funding(self, exchange, fundings):
        return

    def on_funding_fee(self):
        return

    def on_order(self, account_id, order):
        err_receipt = {'Err': {'code': 2000, 'error': 'mock'}}
        for _ in range(5):
            rs = self.risk_manager.check_order_receipt_risk(0, 'BTC_USDT', err_receipt)                 # 检查订单回执风控
        if rs and not rs.is_empty():
            print(f"order 回执风控告警: {rs.to_dict()}")
            print(f"---------------------------------------------------------")
        rs_quality = self.risk_manager.check_order_risk()                 # 检查订单质量风控
        if rs_quality and not rs_quality.is_empty():
            print(f"order 质量风控告警: {rs_quality.to_dict()}")
            print(f"---------------------------------------------------------")
            
    def on_strategy(self, index, symbol):
        return

    def on_balance(self, account_id, balance):
        risk_signal = self.risk_manager.check_global_risk()                 # 检查全局风控
        if risk_signal and not risk_signal.is_empty():
            print(f"balance 风控告警: {risk_signal.to_dict()}")
            print(f"---------------------------------------------------------")
        return

    def on_stop(self):
        return

    def on_trade(self, exchange, trade):
        return

    def on_timer_subscribe(self, timer_name):
        risk_signal = self.risk_manager.check_global_risk()                 # 检查全局风控
        if risk_signal and not risk_signal.is_empty():
            print(f"global 风控告警: {risk_signal.to_dict()}")
            print(f"---------------------------------------------------------")


if __name__ == "__main__":
    mock_strategy = Strategy()
    strategy_methods = sorted([method for method in dir(Strategy) if not method.startswith('_')])  # 按照字母顺序排序
    print(f"Strategy 类的方法: {strategy_methods}")
    print(f"--------------------------------")
    for method in strategy_methods:
        print(f"执行 {method} 方法")
        getattr(mock_strategy, method)()
        print(f"--------------------------------")
        
        
    


import os
import sys
from typing import Dict, Any, Optional
from pathlib import Path

try:
    import tomllib  # Python 3.11+
    _USE_STDLIB_TOML = True
except ImportError:
    try:
        import toml as tomllib  # 使用 toml 库作为后备
        _USE_STDLIB_TOML = False
    except ImportError:
        raise ImportError("需要安装 toml 库: pip install toml")

# 添加项目根目录到路径
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))


class ConfigLoader:
    """
    配置加载器，用于读取和管理 config.toml 配置文件。
    支持文件更新检测和自动重新加载。
    """
    
    _instance: Optional['ConfigLoader'] = None
    _config: Dict[str, Any] = {}
    _config_path: Optional[str] = None
    _last_modified: float = 0.0
    
    def __new__(cls, config_path: Optional[str] = None):
        """
        单例模式，确保全局只有一个配置加载器实例。
        """
        if cls._instance is None:
            cls._instance = super().__new__(cls)
            cls._instance._initialized = False
        return cls._instance
    
    def __init__(self, config_path: Optional[str] = None):
        """
        初始化配置加载器。
        
        Args:
            config_path: 配置文件路径，如果为 None，则使用默认路径 config/config.toml
        """
        if self._initialized:
            return
        
        if config_path is None:
            # 获取项目根目录
            project_root = Path(__file__).parent.parent
            config_path = str(project_root / "config" / "config.toml")
        
        self._config_path = config_path
        self._load_config()
        self._initialized = True
    
    def _load_config(self) -> None:
        """
        从配置文件加载配置，并更新最后修改时间。
        """
        if not os.path.exists(self._config_path):
            raise FileNotFoundError(f"配置文件不存在: {self._config_path}")
        
        try:
            # 根据使用的库选择不同的读取方式
            if _USE_STDLIB_TOML:
                # Python 3.11+ 的 tomllib（标准库）
                with open(self._config_path, 'rb') as f:
                    self._config = tomllib.load(f)
            else:
                # toml 库（第三方库）
                with open(self._config_path, 'r', encoding='utf-8') as f:
                    self._config = tomllib.loads(f.read())
            
            # 更新最后修改时间
            self._last_modified = os.path.getmtime(self._config_path)
        except Exception as e:
            raise RuntimeError(f"加载配置文件失败: {e}")
    
    def _check_and_reload(self) -> None:
        """
        检查配置文件是否有更新，如果有则重新加载。
        """
        if not os.path.exists(self._config_path):
            return
        
        current_modified = os.path.getmtime(self._config_path)
        if current_modified > self._last_modified:
            self._load_config()
    
    def get(self, key: str, default: Any = None) -> Any:
        """
        获取配置值，如果不存在则返回默认值。
        每次获取时都会检查文件是否有更新。
        
        Args:
            key: 配置键名
            default: 默认值
            
        Returns:
            配置值或默认值
        """
        self._check_and_reload()
        return self._config.get(key, default)
    
    def get_tick_config(self) -> Dict[str, Any]:
        """
        获取 TICK 级别的配置。
        
        Returns:
            TICK 级别配置字典
        """
        self._check_and_reload()
        return {
            'abnormal_spread_threshold': self._config.get('abnormal_spread_threshold', 0.2),
            'abnormal_bbo_latency_threshold': self._config.get('abnormal_bbo_latency_threshold', 5000),
        }
    
    def get_order_config(self) -> Dict[str, Any]:
        """
        获取 ORDER 级别的配置。
        
        Returns:
            ORDER 级别配置字典
        """
        self._check_and_reload()
        return {
            'abnormal_order_slippage_threshold': self._config.get('abnormal_order_slippage_threshold', 30.0),
            'abnormal_order_latency_threshold': self._config.get('abnormal_order_latency_threshold', 600),
        }
    
    def get_global_config(self) -> Dict[str, Any]:
        """
        获取 GLOBAL 级别的配置。
        
        Returns:
            GLOBAL 级别配置字典
        """
        self._check_and_reload()
        return {
            'max_account_leverage': self._config.get('max_account_leverage', 8.0),
            'max_symbol_leverage': self._config.get('max_symbol_leverage', 1.0),
            'abnormal_position_exposure_threshold': self._config.get('abnormal_position_exposure_threshold', 0.05),
            'available_balance_warning_threshold': self._config.get('available_balance_warning_threshold', 0.2),
            'available_balance_unbalance_threshold': self._config.get('available_balance_unbalance_threshold', 1.2),
            'total_drawdown_threshold': self._config.get('total_drawdown_threshold', 0.95),
            'intraday_loss_threshold': self._config.get('intraday_loss_threshold', 1000.0),
            'liquidation_price_warning_threshold': self._config.get('liquidation_price_warning_threshold', 0.01),
        }
    
    def reload(self) -> None:
        """
        强制重新加载配置文件。
        """
        self._load_config()


# 全局配置加载器实例
_config_loader: Optional[ConfigLoader] = None


def get_config_loader(config_path: Optional[str] = None) -> ConfigLoader:
    """
    获取全局配置加载器实例。
    
    Args:
        config_path: 配置文件路径，仅在首次调用时生效
        
    Returns:
        配置加载器实例
    """
    global _config_loader
    if _config_loader is None:
        _config_loader = ConfigLoader(config_path)
    return _config_loader




from typing import List, Dict, Any


def position_liquidation_price_check(
    positions: List[List[Dict[str, Any]]],
    bbo: List[Dict[str, Any]],
    liquidation_price_warning_threshold: float = 0.01,
) -> bool:
    """
    检查所有 position 当前 symbol 的价格是否接近强制平仓价格。
    position 是 [[{}] [{}]] 来存储全部交易所的 全部 position
    需要先遍历交易所，再遍历 position，再获取 bbo 数据
    """
    result = {'Info': dict()}

    exchange_count = len(positions)

    for exchange in range(exchange_count):
        for position in positions[exchange]:
            symbol = position['symbol']
            mid_price = (bbo[exchange][symbol][0] + bbo[exchange][symbol][2]) / 2
            if (
                position['liquidation_price'] != 0 and abs(position['liquidation_price'] - mid_price) / position['liquidation_price']
                > liquidation_price_warning_threshold
            ):
                result['Info'][symbol] = {
                    'liquidation_price': position['liquidation_price'],
                    'mid_price': mid_price,
                }

    return result


import unittest
import sys
import os
sys.path.append(os.path.dirname(os.path.abspath(__file__)) + '/../')
from typing import Any, Dict

from risk_manager import RiskManager


class TestOrderQualityRisk(unittest.TestCase):
    def setUp(self):
        self.rm = RiskManager()
        self.base = 1_000_000_000_000
        self.now = self.base
        self.rm._now_ms = lambda: self.now
        self.rm.ex_map = {0: "BinanceSwap", 1: "OkxSwap", "BinanceSwap": 0, "OkxSwap": 1}
        self.symbol = "BTC_USDT"
        self.exchange = "BINANCE"

    def make_order(self, ts_offset_ms: int, price: float, filled_avg_price: float) -> Dict[str, Any]:
        return {
            "exchange": self.exchange,
            "symbol": self.symbol,
            "timestamp": self.base + ts_offset_ms,
            "price": price,
            "filled": 10.0,
            "filled_avg_price": filled_avg_price,
        }

    def test_slippage_window_trigger(self):
        # 10秒内滑点>15bps累计5次后触发
        for i in range(4):
            self.rm.latest_order = self.make_order(0, 100.0, 100.3)  # 30 bps
            rs = self.rm.check_order_risk()
            self.assertTrue(rs.is_empty())
            self.now += 1000
        self.rm.latest_order = self.make_order(0, 100.0, 100.3)
        rs = self.rm.check_order_risk()
        self.assertFalse(rs.is_empty())

    def test_latency_window_trigger(self):
        # 10秒内延迟>800ms累计5次后触发
        for i in range(4):
            self.rm.latest_order = {
                "exchange": self.exchange,
                "symbol": self.symbol,
                "timestamp": self.base - 2000,  # 让延迟>800ms
                "price": 100.0,
                "filled": 0.0,
            }
            rs = self.rm.check_order_risk()
            self.assertTrue(rs.is_empty())
            self.now += 1000
        self.rm.latest_order = {
            "exchange": self.exchange,
            "symbol": self.symbol,
            "timestamp": self.base - 2000,
            "price": 100.0,
            "filled": 0.0,
        }
        rs = self.rm.check_order_risk()
        self.assertFalse(rs.is_empty())


if __name__ == "__main__":
    unittest.main()





from enum import Enum

class RiskScope(Enum):
    TICK = "tick"
    ORDER = "order"
    GLOBAL = "global"
    SYMBOL = "symbol"


class RiskAction(Enum):
    NONE = "none"
    STOP_OPEN = "stop_open"
    STOP_CLOSE = "stop_close"
    STOP_ALL = "stop_all"
    CLOSE_POSITION = "close_position"


__all__ = ["RiskScope", "RiskAction"]





import functools
import inspect
import time

from datetime import datetime
from typing import Any, Callable, Dict, Optional
from utils.position_manager import HedgePositionManager
from utils.statistics import Statistics
from utils.signal_manager import SignalManager
from utils.risk_signal import RiskSignal
from enums.risk_signal_types import RiskScope, RiskAction
from usecase.order_level.exchange_order_check import (
    exchange_order_slippage_check,
    exchange_order_latency_check,
    exchange_order_failure_check,
)
from usecase.order_level.symbol_order_check import (
    symbol_order_slippage_check,
    symbol_order_latency_check,
    symbol_order_failure_check,
)
from usecase.tick_level.spread_check import (
    spread_threshold_check,
    spread_price_amount_abnormal_check,
)
from usecase.tick_level.latency_check import bbo_latency_check
from usecase.global_level.balance_check import (
    available_balance_ratio_check,
    balance_zero_check,
    balance_unbalance_check,
    balance_drawdown_check,
    intraday_profit_threshold_check,
)
from usecase.global_level.exposure_check import (
    symbol_exposure_check,
    symbol_single_leg_check,
    symbol_notional_exposure_check
)
from usecase.global_level.leverage_check import leverage_check
from usecase.global_level.position_check import position_liquidation_price_check
from usecase.global_level.commission_check import commission_check

__version__ = "0.1.8"


class RiskManager:
    """
    风控信号管理器，用于定义风控信号检查的和内容。
    每一个风控信号应该由以下内容组成：
    - symbol 层：对单独 symbol 进行风控的检查
    - exchange 层：对单独交易所进行风控的检查
    - account 层：对单独账户进行风控的检查
    - global 层：对全局进行风控的检查
    对于不同层级的解释如下：
    - symbol 层：单独监控某一个 symbol 的行情、持仓、PnL 异常等情况并进行风控。
    - exchange 层：单独监控某一个或者几个交易所发生异常行为，例如交易所拔网线等。
    - account 层：单独监控整个账户的余额、持仓、PnL 的异常并进行风控。例如账户可用资金为 0, 账户可用资金不足 20% 等。
    - global 层：对全局进行风控的检查。例如账户回撤超过 5%，账户可用资金超过 x 秒没有被平衡等。
    当用户调用风控结果的时候，最终返回一个 RiskSignal 对象。如果一切正常，则返回的是一个空 RiskSignal 对象。通过对风控信号解析可以获取对应的风控内容
    """

    def __init__(
        self,
        abnormal_spread_threshold: float = 0.2,
        abnormal_bbo_latency_threshold: int = 5000,
        abnormal_order_slippage_threshold: float = 30.0,
        abnormal_order_latency_threshold: int = 600,
        max_account_leverage: float = 8.0,
        max_symbol_leverage: float = 1.0,
        abnormal_position_exposure_threshold: float = 0.05,
        available_balance_warning_threshold: float = 0.2,
        available_balance_unbalance_threshold: float = 1.2,
        total_drawdown_threshold: float = 0.95,
        intraday_loss_threshold: float = 1000.0,
        liquidation_price_warning_threshold: float = 0.01,
    ):
        """
        风控管理器
        - 自动绑定并缓存 Strategy 成员
        - 缓存并追踪关键字段: bbo / depth / funding / order / pm
        - 缓存风控配置参数（阈值）
        """
        # 缓存风控配置参数（阈值）
        self.abnormal_spread_threshold = abnormal_spread_threshold
        self.abnormal_bbo_latency_threshold = abnormal_bbo_latency_threshold
        self.abnormal_order_slippage_threshold = abnormal_order_slippage_threshold
        self.abnormal_order_latency_threshold = abnormal_order_latency_threshold
        self.max_account_leverage = max_account_leverage
        self.max_symbol_leverage = max_symbol_leverage
        self.abnormal_position_exposure_threshold = abnormal_position_exposure_threshold
        self.available_balance_warning_threshold = available_balance_warning_threshold
        self.available_balance_unbalance_threshold = available_balance_unbalance_threshold
        self.total_drawdown_threshold = total_drawdown_threshold
        self.intraday_loss_threshold = intraday_loss_threshold
        self.liquidation_price_warning_threshold = liquidation_price_warning_threshold

        self._strategy: Optional[Any] = None
        self.cached_attrs: Dict[str, Any] = {}
        self.bbo: Any = None
        self.fundings: Any = None
        self.pm: Any = None
        self.ex_map: Any = None
        self.balances: Any = None
        self.trader: Any = None
        self.previous_commission: Optional[Any] = None

        # on_* 方法参数本地缓存。用于识别行情异常、订单异常、全局异常。
        self._last_signal: Optional[Dict[str, Any]] = None
        # on_* 参数本地缓存（仅针对指定的方法）
        self.latest_bbo: Any = None
        self.latest_order: Any = None
        self.latest_order_account_id: Optional[Any] = None
        self.latest_strategy_index: Optional[Any] = None
        self.latest_strategy_symbol: Optional[Any] = None
        self.latest_commission: Optional[Any] = None

        # 本地订单状态统计（时间窗口触发）
        # 订单状态统计分为：exchange 和 symbol 两个层级。窗口为 10 秒，阈值为 5 次。
        self._win_receipt_ex: Dict[str, Dict[str, int]] = {}
        self._win_receipt_sym: Dict[str, Dict[str, int]] = {}
        self._win_slip_ex: Dict[str, Dict[str, int]] = {}
        self._win_slip_sym: Dict[str, Dict[str, int]] = {}
        self._win_lat_ex: Dict[str, Dict[str, int]] = {}
        self._win_lat_sym: Dict[str, Dict[str, int]] = {}


    def run(self):
        """
        还没想好，这个方法用来做什么，但是应该需要
        """
        return

    def check_tick_risk(self) -> RiskSignal:
        """
        检查 tick 级别的风控，需要传入 bbo 数据或者 depth 数据，并不一定都要传入。实现以下风控功能：
        - 行情价差异常监控
            - symbol 相同，价差超过 20%
            - symbol 价格本身有问题，例如 bid1 == ask1，又或者 0 价格和 amount
            - symbol 的行情距离当前时间太远，被交易所拔网线
        - 行情延迟风控
        """
        if not self.latest_bbo or 'symbol' not in self.latest_bbo:
            return RiskSignal()
        symbol = self.latest_bbo['symbol']
        if symbol not in self.bbo[0] or symbol not in self.bbo[1]:
            return RiskSignal()
        check_bbo = [self.bbo[0][symbol], self.bbo[1][symbol]]
        spread_check = spread_threshold_check(
            check_bbo,
            abnormal_spread_threshold=self.abnormal_spread_threshold,
        )
        price_amount_abnormal_check = spread_price_amount_abnormal_check(check_bbo)
        latency_check = bbo_latency_check(
            check_bbo,
            abnormal_bbo_latency_threshold=self.abnormal_bbo_latency_threshold,
        )

        # 生成信号并返回
        # 所有检查函数返回 True 表示正常，False 表示异常
        rs = RiskSignal()
        if not spread_check:
            # 价差异常，禁止该 symbol 开仓
            rs.add(
                scope=RiskScope.TICK,
                action=RiskAction.STOP_ALL,
                message="价差异常，禁止该 symbol 开仓",
                target_id=symbol,
            )
        if not price_amount_abnormal_check:
            # 交易所的价格和数量异常，禁止该 symbol 交易
            rs.add(
                scope=RiskScope.TICK,
                action=RiskAction.STOP_ALL,
                message="交易所的价格和数量异常，禁止该 symbol 交易",
                target_id=symbol,
            )
        if not latency_check:
            # 交易所的行情延迟异常，禁止该 symbol 交易
            rs.add(
                scope=RiskScope.TICK,
                action=RiskAction.STOP_ALL,
                message="交易所的行情延迟异常，禁止该 symbol 交易",
                target_id=symbol,
            )
        
        self._last_signal = rs.to_dict()
        return rs

    def check_order_receipt_risk(self, index: int, symbol: str, order_receipt: Dict[str, Any]) -> RiskSignal:
        """
        订单回执风控：当相同交易所/相同symbol第一次检测到失败，创建10秒窗口；
        10秒内相同错误累计>=5次则生成 RiskSignal；否则过期销毁窗口。
        失败定义：order_receipt 中不包含 'Ok' 字段。
        """
        rs = RiskSignal()
        # 推断交易所标识
        try:
            exchange = str(self.ex_map[index]) if isinstance(self.ex_map, dict) and index in self.ex_map else str(index)
        except Exception:
            exchange = str(index)
        is_failure = isinstance(order_receipt, dict) and ("Ok" not in order_receipt)
        if not is_failure:
            return rs
        window_ms = 10000
        min_hits = 5
        # 交易所窗口
        if self._bump_window(self._win_receipt_ex, exchange, window_ms, min_hits):
            rs.add(
                scope=RiskScope.ORDER,
                action=RiskAction.STOP_ALL,
                message="交易所层面：10秒内订单回执失败达到阈值，停止所有交易",
                target_id=exchange,
                context={"window_ms": window_ms, "min_hits": min_hits},
            )
        # Symbol 窗口
        if self._bump_window(self._win_receipt_sym, symbol, window_ms, min_hits):
            rs.add(
                scope=RiskScope.ORDER,
                action=RiskAction.STOP_ALL,
                message="Symbol 层面：10秒内订单回执失败达到阈值，停止该Symbol交易",
                target_id=symbol,
                context={"window_ms": window_ms, "min_hits": min_hits},
            )
        self._last_signal = rs.to_dict()
        return rs

    def check_order_risk(self) -> RiskSignal:
        """
        订单质量风控（10秒窗口累计）：
        - 首次发现：滑点 > 0.15%（15 bps）或 下单延迟 > 800ms -> 启动窗口
        - 10秒内相同错误累计 >= 5 次 -> 生成 RiskSignal；否则窗口过期销毁
        """
        rs = RiskSignal()
        if not isinstance(self.latest_order, dict):
            self._last_signal = rs.to_dict()
            return rs
        symbol = str(self.latest_order.get("symbol", "unknown"))
        exchange = str(self.latest_order.get("exchange", "unknown"))

        window_ms = 10_000
        min_hits = 5
        # 使用 RiskManager 中缓存的阈值
        ex_slip_bad, ex_slip_bps, _ = exchange_order_slippage_check(
            self.latest_order,
            threshold_bps=self.abnormal_order_slippage_threshold,
        )
        ex_lat_bad, ex_lat_ms, _ = exchange_order_latency_check(
            self.latest_order,
            threshold_ms=self.abnormal_order_latency_threshold,
        )
        sym_slip_bad, sym_slip_bps, _ = symbol_order_slippage_check(
            symbol,
            self.latest_order,
            threshold_bps=self.abnormal_order_slippage_threshold,
        )
        sym_lat_bad, sym_lat_ms, _ = symbol_order_latency_check(
            symbol,
            self.latest_order,
            threshold_ms=self.abnormal_order_latency_threshold,
        )

        # 交易所层面窗口计数
        if ex_slip_bad and self._bump_window(self._win_slip_ex, exchange, window_ms, min_hits):
            rs.add(
                scope=RiskScope.ORDER,
                action=RiskAction.STOP_ALL,
                message="交易所层面：10秒内滑点超阈值达到次数阈值，暂停交易",
                target_id=exchange,
                context={"window_ms": window_ms, "min_hits": min_hits, "slippage_bps": ex_slip_bps},
            )
        if ex_lat_bad and self._bump_window(self._win_lat_ex, exchange, window_ms, min_hits):
            rs.add(
                scope=RiskScope.ORDER,
                action=RiskAction.STOP_ALL,
                message="交易所层面：10秒内下单延迟超阈值达到次数阈值，暂停交易",
                target_id=exchange,
                context={"window_ms": window_ms, "min_hits": min_hits, "latency_ms": ex_lat_ms},
            )

        # Symbol 层面窗口计数
        if sym_slip_bad and self._bump_window(self._win_slip_sym, symbol, window_ms, min_hits):
            rs.add(
                scope=RiskScope.ORDER,
                action=RiskAction.STOP_ALL,
                message="Symbol 层面：10秒内滑点超阈值达到次数阈值，暂停交易",
                target_id=symbol,
                context={"window_ms": window_ms, "min_hits": min_hits, "slippage_bps": sym_slip_bps},
            )
        if sym_lat_bad and self._bump_window(self._win_lat_sym, symbol, window_ms, min_hits):
            rs.add(
                scope=RiskScope.ORDER,
                action=RiskAction.STOP_ALL,
                message="Symbol 层面：10秒内下单延迟超阈值达到次数阈值，暂停交易",
                target_id=symbol,
                context={"window_ms": window_ms, "min_hits": min_hits, "latency_ms": sym_lat_ms},
            )

        self._last_signal = rs.to_dict()
        return rs

    def check_global_risk(self) -> RiskSignal:
        """
        检查全局风控，实现了以下风控功能：
        - 整体账户余额回撤
        - 当日收益超过阈值后禁止开仓，只可以平仓
        - 可用资金为 0 风控
        - 账户可用资金风控
        - 账户杠杆风控
        - 账户可用资金不平衡风控
        - 单个 symbol 杠杆率过高风控
        - symbol 敞口风控
        - symbol 单腿风控
        """
        # 需要使用以下数据：self.balances, self.statistics, self.pm
        # - 使用 self.balances 完成需求：可用资金为 0 风控；账户可用资金风控；账户可用资金不平衡风控。
        # - 使用 self.statistics 完成需求：整体账户余额回撤; 当日收益超过阈值禁止开仓；
        # - 使用 self.pm 完成需求：账户杠杆风控；单个 symbol 杠杆率过高风控; symbol 敞口风控; symbol 单腿风控。
        balance_ratio_check = available_balance_ratio_check(
            self.balances,
            available_balance_warning_threshold=self.available_balance_warning_threshold,
        )
        zero_check = balance_zero_check(self.balances)
        unbalance_check = balance_unbalance_check(
            self.balances,
            available_balance_unbalance_threshold=self.available_balance_unbalance_threshold,
        )

        # 以下的用例检查，需要引入 self.statistics 来实现
        drawdown_check = balance_drawdown_check(
            self.statistics,
            total_drawdown_threshold=self.total_drawdown_threshold,
        )
        profit_threshold_check = intraday_profit_threshold_check(
            self.statistics,
            intraday_loss_threshold=self.intraday_loss_threshold,
        )

        exposure_check = symbol_exposure_check(
            self.symbols[0],
            self.pm,
            abnormal_position_exposure_threshold=self.abnormal_position_exposure_threshold,
        )
        notional_exposure_check = symbol_notional_exposure_check(
            self.symbols[0],
            self.bbo,
            self.pm,
            abnormal_position_exposure_threshold=self.abnormal_position_exposure_threshold,
        )
        single_leg_check = symbol_single_leg_check(self.symbols[0], self.pm)

        # 杠杆的判断需要用 balance 、 bbo 和  position manager 合起来判断。
        leverage_result = leverage_check(
            self.symbols[0],
            self.bbo,
            self.balances,
            self.pm,
            max_symbol_leverage=self.max_symbol_leverage,
            max_account_leverage=self.max_account_leverage,
        )
        # leverage_result 是一个字典：{'account_leverage': False, 'symbol_leverage': {}}
        # account_leverage 为 True 表示账户杠杆过高，False 表示正常
        # symbol_leverage 是一个字典，key 是 symbol，value 是 True/False
        leverage_check_passed = not leverage_result.get('account_leverage', False) and len(leverage_result.get('symbol_leverage', {})) == 0

        liquidation_price_check = position_liquidation_price_check(
            self.positions,
            self.bbo,
            liquidation_price_warning_threshold=self.liquidation_price_warning_threshold,
        )

        # 使用 self.trader 来获取手续费等级，和之前的费率进行对比。
        # 使用第一个账户的索引，或者使用 latest_order_account_id（如果存在）
        account_idx = self.latest_order_account_id if self.latest_order_account_id is not None else 0
        # fee_rate = self.trader.get_fee_rate(account_idx, "BTC_USDT")
        fee_rate = {'buyer': 0.0, 'maker': 0.0002, 'seller': 0.0, 'taker': 0.0005}
        if not self.previous_commission:
            self.previous_commission = fee_rate
            commission_check_result = True
        else:
            commission_check_result = commission_check(self.previous_commission, fee_rate)

        if balance_ratio_check and zero_check and unbalance_check \
           and drawdown_check and profit_threshold_check and not exposure_check['exposure'] \
           and not notional_exposure_check['exposure'] and single_leg_check and leverage_check_passed \
           and commission_check_result and not liquidation_price_check['Info']:
            rs = RiskSignal()
            self._last_signal = rs.to_dict()
            return rs
        else:
            rs = RiskSignal()
            if not balance_ratio_check:
                rs.add(
                    scope=RiskScope.GLOBAL,
                    action=RiskAction.STOP_OPEN,
                    message="可用资金比例异常，暂停开仓",
                    target_id="ALL",
                )
                self._last_signal = rs.to_dict()
            if not zero_check:
                rs.add(
                    scope=RiskScope.GLOBAL,
                    action=RiskAction.STOP_OPEN,
                    message="账户资金为 0，暂停开仓",
                    target_id="ALL",
                )
                self._last_signal = rs.to_dict()
            if not unbalance_check:
                rs.add(
                    scope=RiskScope.GLOBAL,
                    action=RiskAction.STOP_OPEN,
                    message="账户资金不平衡，暂停开仓",
                    target_id="ALL",
                )
                self._last_signal = rs.to_dict()
            if not drawdown_check:
                rs.add(
                    scope=RiskScope.GLOBAL,
                    action=RiskAction.STOP_OPEN,
                    message="账户资金回撤过大，暂停开仓",                    target_id="ALL",
                )
                self._last_signal = rs.to_dict()
            if not profit_threshold_check:
                rs.add(
                    scope=RiskScope.GLOBAL,
                    action=RiskAction.STOP_OPEN,
                    message="当日收益超过阈值，暂停开仓",
                    target_id="ALL",
                )
                self._last_signal = rs.to_dict()
            if exposure_check['exposure']:
                rs.add(
                    scope=RiskScope.SYMBOL,
                    action=RiskAction.CLOSE_POSITION,
                    message="symbol 敞口过大，关闭该 symbol 敞口",
                    target_id=exposure_check['exposure'],
                )
                self._last_signal = rs.to_dict()
            if notional_exposure_check['exposure']:
                rs.add(
                    scope=RiskScope.SYMBOL,
                    action=RiskAction.CLOSE_POSITION,
                    message="symbol 名义市值敞口过大，关闭该 symbol 敞口",
                    target_id=notional_exposure_check['exposure'],
                )
                self._last_signal = rs.to_dict()
            if not single_leg_check:
                rs.add(
                    scope=RiskScope.GLOBAL,
                    action=RiskAction.CLOSE_POSITION, 
                    message="symbol 单腿风控，关闭该 symbol 单腿",
                    target_id="ALL",
                )
                self._last_signal = rs.to_dict()
            if not leverage_check_passed:
                # 检查账户杠杆
                if leverage_result.get('account_leverage', False):
                    rs.add(
                        scope=RiskScope.GLOBAL,
                        action=RiskAction.STOP_OPEN,
                        message="账户杠杆过高，暂停开仓",
                        target_id="ALL",
                    )
                # 检查 symbol 杠杆
                for symbol, is_high_leverage in leverage_result.get('symbol_leverage', {}).items():
                    if is_high_leverage:
                        rs.add(
                            scope=RiskScope.SYMBOL,
                            action=RiskAction.STOP_OPEN,
                            message=f"symbol {symbol} 杠杆过高，暂停该 symbol 交易",
                            target_id=symbol,
                        )
                self._last_signal = rs.to_dict()
            if not commission_check_result:
                rs.add(
                    scope=RiskScope.GLOBAL,
                    action=RiskAction.STOP_ALL,
                    message="交易所手续费发生变化，暂停所有交易",
                    target_id="ALL",
                )
                self._last_signal = rs.to_dict()
            if liquidation_price_check['Info']:
                rs.add(
                    scope=RiskScope.SYMBOL,
                    action=RiskAction.CLOSE_POSITION,
                    message="position 强制平仓价格异常，关闭该 position",
                    target_id=liquidation_price_check['Info'].keys(),
                    context={"liquidation_price_check": liquidation_price_check['Info']},
                )
                self._last_signal = rs.to_dict()
        return rs

    def bind_strategy(self, strategy_instance: Any) -> None:
        """
        绑定策略实例，并初始化一次成员快照与关键字段缓存。
        """
        self._strategy = strategy_instance
        self._snapshot_strategy_members()
        self.update_from_strategy()

    def update_from_strategy(self) -> None:
        """
        从绑定的 Strategy 实例同步关键字段与成员快照。
        将策略所有的成员变量都缓存到该 RiskManager 实例中
        """
        if self._strategy is None:
            return
        # 缓存关键字段
        self.base_index = getattr(self._strategy, "base_index", 2)
        self.bbo = getattr(self._strategy, "bbo", [{} for _ in range(self.base_index)])
        self.fundings = getattr(self._strategy, "fundings", None)
        self.pm = getattr(self._strategy, "pm", None)
        self.ex_map = getattr(self._strategy, "ex_map", None)
        self.balances = getattr(self._strategy, "balances", None)
        self.trader = getattr(self._strategy, "trader", None)
        self.positions = getattr(self._strategy, "positions", [[], []])
        self.depth = getattr(self._strategy, "depth", [{} for _ in range(self.base_index)])
        self.symbols = getattr(self._strategy, "symbols", None)
        self.statistics = getattr(self._strategy, "statistics", Statistics())
        # 刷新所有非可调用的成员快照
        self._snapshot_strategy_members()

    def _cache_on_event(self, method_name: str, args: tuple, kwargs: dict) -> None:
        """
        根据 on_* 方法名将传入参数缓存到 RiskManager。
        仅缓存以下方法：
        - on_bbo(self, exchange, bbo)
        - on_depth(self, exchange, depth)
        - on_order(self, account_id, order)
        - on_strategy(self, index, symbol)
        - on_balance(self, account_id, balance)
        - on_funding(self, exchange, funding)
        并且根据新接收到的数据，对 self.bbo 、self.depth 、self.balances 和 self.fundings进行更新
        """
        if method_name == "on_bbo":
            exchange = kwargs.get("exchange", args[0] if len(args) > 0 else None)
            bbo = kwargs.get("bbo", args[1] if len(args) > 1 else None)
            index = self.ex_map[exchange]
            self.bbo[index][bbo.get('symbol', '')] = bbo
            self.latest_bbo = bbo
        elif method_name == "on_depth":
            exchange = kwargs.get("exchange", args[0] if len(args) > 0 else None)
            depth = kwargs.get("depth", args[1] if len(args) > 1 else None)
            index = self.ex_map[exchange]
            self.depth[index][depth.get('symbol', '')] = depth
            latest_bbo = {
                'symbol': depth.get('symbol', ''),
                'bid_price': depth['bids'][0][0],
                'bid_qty': depth['bids'][0][1],
                'ask_price': depth['asks'][0][0],
                'ask_qty': depth['asks'][0][1],
                'timestamp': depth['timestamp']
            }
            self.bbo[index][depth.get('symbol', '')] = latest_bbo
            self.latest_bbo = latest_bbo
        elif method_name == "on_order":
            account_id = kwargs.get("account_id", args[0] if len(args) > 0 else None)
            order = kwargs.get("order", args[1] if len(args) > 1 else None)
            self.latest_order_account_id = account_id
            self.latest_order = order
        elif method_name == "on_strategy":
            index = kwargs.get("index", args[0] if len(args) > 0 else None)
            symbol = kwargs.get("symbol", args[1] if len(args) > 1 else None)
            self.latest_strategy_index = index
            self.latest_strategy_symbol = symbol
        elif method_name == "on_balance":
            account_id = kwargs.get("account_id", args[0] if len(args) > 0 else None)
            balances = kwargs.get("balances", args[1] if len(args) > 1 else None)
            usdt = next((x for x in balances if x["asset"] == "USDT"), None)
            self.balances[account_id].update(usdt)
        elif method_name == "on_funding":
            exchange = kwargs.get("exchange", args[0] if len(args) > 0 else None)
            funding = kwargs.get("funding", args[1] if len(args) > 1 else None)
            index = self.ex_map[exchange]
            for item in funding:
                self.fundings[index][item.get('symbol', '')] = item

    def _now_ms(self) -> int:
        return int(time.time() * 1000)

    def _bump_window(self, store: Dict[str, Dict[str, int]], key: str, window_ms: int, min_hits: int) -> bool:
        """
        在 store[key] 上进行 10 秒窗口计数：达到 min_hits 返回 True，并清空该窗口；否则返回 False。
        规则：
        - 第一次命中：创建窗口 {start, count=1}
        - 窗口内再次命中：count+1；若 count>=min_hits -> 返回 True 并删除窗口
        - 超过窗口时间且未达阈值：销毁旧窗口，重新从1开始
        """
        now = self._now_ms()
        entry = store.get(key)
        if entry is None or (now - entry.get("start", 0)) > window_ms:
            store[key] = {"start": now, "count": 1}
            return False
        entry["count"] = entry.get("count", 0) + 1
        if entry["count"] >= min_hits:
            try:
                del store[key]
            except Exception:
                store[key] = {"start": now, "count": 0}
            return True
        return False

    def _snapshot_strategy_members(self) -> None:
        """
        抓取 Strategy 的所有"数据型"成员变量（排除可调用与双下划线）。
        TODO: 这样能够将 Strategy 在 init 方法中的变量都缓存，但是需要考虑有一些变量是在 start() 和 prepare_symbols() 方法中初始化的，这些变量也需要缓存
        """
        if self._strategy is None:
            return
        result: Dict[str, Any] = {}
        for name in dir(self._strategy):
            if name.startswith("__"):
                continue
            try:
                value = getattr(self._strategy, name)
            except Exception:
                continue
            if callable(value):
                continue
            result[name] = value
        self.cached_attrs = result


def risk_managed(
    abnormal_spread_threshold: float = 0.2,
    abnormal_bbo_latency_threshold: int = 5000,
    abnormal_order_slippage_threshold: float = 30.0,
    abnormal_order_latency_threshold: int = 600,
    max_account_leverage: float = 8.0,
    max_symbol_leverage: float = 1.0,
    abnormal_position_exposure_threshold: float = 0.05,
    available_balance_warning_threshold: float = 0.2,
    available_balance_unbalance_threshold: float = 1.2,
    total_drawdown_threshold: float = 0.95,
    intraday_loss_threshold: float = 1000.0,
    liquidation_price_warning_threshold: float = 0.01,
    on_prefix: str = "on_",
) -> Callable[[type], type]:
    """
    类装饰器：为 Strategy 自动注入 RiskManager，并包装所有以 on_ 开头的方法。
    - 在每次 on_ 方法调用前：同步 bbo/fundings/pm，执行 risk_check
    - 在实例上注入 self.risk_manager 以供直接访问 get_risk_signal()
    """
    def decorator(cls: type) -> type:
        # 包装 __init__：创建并绑定 RiskManager
        orig_init = getattr(cls, "__init__", None)

        @functools.wraps(orig_init)
        def __init__(self, *args, **kwargs):
            if orig_init is not None:
                orig_init(self, *args, **kwargs)
            else:
                super(cls, self).__init__(*args, **kwargs)  # 如果没有触发初始化，理论兜底再进行初始化
            rm = RiskManager(
                abnormal_spread_threshold=abnormal_spread_threshold,
                abnormal_bbo_latency_threshold=abnormal_bbo_latency_threshold,
                abnormal_order_slippage_threshold=abnormal_order_slippage_threshold,
                abnormal_order_latency_threshold=abnormal_order_latency_threshold,
                max_account_leverage=max_account_leverage,
                max_symbol_leverage=max_symbol_leverage,
                abnormal_position_exposure_threshold=abnormal_position_exposure_threshold,
                available_balance_warning_threshold=available_balance_warning_threshold,
                available_balance_unbalance_threshold=available_balance_unbalance_threshold,
                total_drawdown_threshold=total_drawdown_threshold,
                intraday_loss_threshold=intraday_loss_threshold,
                liquidation_price_warning_threshold=liquidation_price_warning_threshold,
            )
            setattr(self, "risk_manager", rm)
            rm.bind_strategy(self)

        setattr(cls, "__init__", __init__)

        # 包装所有 on_ 前缀的方法
        for name, attr in list(vars(cls).items()):
            if not name.startswith(on_prefix):
                continue
            if not callable(attr):
                continue

            def make_wrapper(method: Callable[..., Any]) -> Callable[..., Any]:
                method_name = method.__name__
                @functools.wraps(method)
                def wrapped(self, *args, **kwargs):
                    rm: Optional[RiskManager] = getattr(self, "risk_manager", None)
                    if rm is None:
                        rm = RiskManager(
                            abnormal_spread_threshold=abnormal_spread_threshold,
                            abnormal_bbo_latency_threshold=abnormal_bbo_latency_threshold,
                            abnormal_order_slippage_threshold=abnormal_order_slippage_threshold,
                            abnormal_order_latency_threshold=abnormal_order_latency_threshold,
                            max_account_leverage=max_account_leverage,
                            max_symbol_leverage=max_symbol_leverage,
                            abnormal_position_exposure_threshold=abnormal_position_exposure_threshold,
                            available_balance_warning_threshold=available_balance_warning_threshold,
                            available_balance_unbalance_threshold=available_balance_unbalance_threshold,
                            total_drawdown_threshold=total_drawdown_threshold,
                            intraday_loss_threshold=intraday_loss_threshold,
                            liquidation_price_warning_threshold=liquidation_price_warning_threshold,
                        )
                        setattr(self, "risk_manager", rm)
                        rm.bind_strategy(self)
                    # 仅针对指定 on_* 方法进行本地参数缓存
                    if method_name in {"on_bbo", "on_depth", "on_order", "on_strategy", "on_balance", "on_funding"}:
                        rm._cache_on_event(method_name, args, kwargs)
                    # 执行数据更新
                    rm.update_from_strategy()
                    return method(self, *args, **kwargs)
                return wrapped

            setattr(cls, name, make_wrapper(attr))

        return cls

    return decorator

if __name__ == "__main__":
    # 新创建一个 Mock Strategy 类用于仿真
    # 最小可运行与功能测试
    @risk_managed()                                           # 对策略类进行装饰，自动注入 RiskManager，并包装所有以 on_ 开头的方法，这种方法可以减少对策略类代码的注入
    class Strategy:
        def __init__(self):
            # 模拟策略的成员变量
            self.bbo = [{'BTC_USDT':{'bid_price':100, 'bid_qty':100, 'ask_price':101, 'ask_qty':100, 'timestamp':1715769600}}, {'BTC_USDT':{'bid_price':99, 'bid_qty':200, 'ask_price':102, 'ask_qty':200, 'timestamp':1715769600}}]
            self.symbols = [["BTC_USDT"], ["BTC_USDT"]]
            self.fundings = [
                {symbol: {"funding_rate": 0, "funding_interval": 8,
                            "next_funding_at": 0} for symbol in self.symbols[idx]}
                for idx in range(len(self.symbols))
            ]
            self.ex_map = {'BinanceSwap': 0, 'OkxSwap': 1, 0: 'BinanceSwap', 1: 'OkxSwap'}
            self.balances = [{}, {}]
            self.pm = HedgePositionManager()
            self.statistics = Statistics()
            self.signal_manager = SignalManager()
            self.name = "MockStrategy"

        def on_bbo(self, exchange, bbo):
            print("on_bbo: 业务逻辑执行")
            index = 1   # 模拟通过 exchange 转为 index 的逻辑
            symbol = bbo.get('symbol', '')
            self.bbo[index][symbol] = {
                'bid_price': bbo.get('bid_price', 0),
                'bid_qty': bbo.get('bid_qty', 0),
                'ask_price': bbo.get('ask_price', 0),
                'ask_qty': bbo.get('ask_qty', 0),
                'timestamp': bbo.get('timestamp', 0)
            }

        def on_depth(self, exchange, depth):
            print("on_depth: 业务逻辑执行")
            index = 1   # 模拟通过 exchange 转为 index 的逻辑
            symbol = depth.get('symbol', '')
            self.bbo[index][symbol] = {
                'bid_price': depth['bids'][0][0],
                'bid_qty': depth['bids'][0][1],
                'ask_price': depth['asks'][0][0],
                'ask_qty': depth['asks'][0][1],
                'timestamp': depth['timestamp']
            }

        def on_balance(self, account_id, balance):
            print("on_balance: 业务逻辑执行")
            self.balances[account_id] = balance

        def on_order(self, account_id, order):
            print("on_order: 业务逻辑执行")

        def on_funding(self, exchange, funding):
            print("on_funding: 业务逻辑执行")

        def on_strategy(self, index, symbol):
            print("on_strategy: 业务逻辑执行")

        def not_triggered(self):
            print(f"{self.__class__.__name__} 由于该方法不以 on_ 开头，该方法不会触发风控")


    # ==================================== 以下是模拟真实场景，对 strategy 进行调用 ==================================
    s = Strategy()
    print("策略初始化后, RiskManager 风控的参数变量更新：")
    print("=== 测试循环更新 self.bbo 数据, 模拟真实场景收到 bbo 数据风控功能是否能正常运行 ===")
    
    # 模拟循环接收到 bbo 数据的真实场景，检查 RiskManager 是否正确同步
    for i in range(5):
        # 更新 Strategy 的 bbo 数据
        print(f"-------------------------------- 第 {i+1} 次更新 bbo 数据, 模拟真实场景收到 bbo 数据 --------------------------------")

        # 调用 on_ 开头的函数触发数据更新
        s.on_bbo('BinanceSwap', {'symbol': 'BTC_USDT', 'bid_price': 100 + i * 1, 'bid_qty': 100 + i * 1, 'ask_price': 101 + i * 1, 'ask_qty': 100 + i * 1, 'timestamp': 1715769600})
    
    print(f"-------------------------------- 开始执行策略 on_strategy 方法 --------------------------------")
    s.on_strategy(0, 'BTC_USDT')
    print(f"-------------------------------- 策略执行完成 --------------------------------")



import unittest

try:
    from usecase.global_level.balance_check import (
        available_balance_ratio_check,
        balance_zero_check,
        balance_unbalance_check,
        balance_drawdown_check,
        intraday_profit_threshold_check,
    )
    _balance_import_error = None
except Exception as e:
    available_balance_ratio_check = None
    balance_zero_check = None
    balance_unbalance_check = None
    balance_drawdown_check = None
    intraday_profit_threshold_check = None
    _balance_import_error = e

try:
    from usecase.global_level.leverage_check import leverage_check
    _leverage_import_error = None
except Exception as e:
    leverage_check = None
    _leverage_import_error = e

try:
    from usecase.global_level.exposure_check import (
        symbol_exposure_check,
        symbol_single_leg_check,
    )
    _exposure_import_error = None
except Exception as e:
    symbol_exposure_check = None
    symbol_single_leg_check = None
    _exposure_import_error = e


class DummyPos:
    def __init__(self, size: float):
        self.size = size


class DummyPM:
    def __init__(self, sizes0, sizes1):
        self.sizes0 = sizes0
        self.sizes1 = sizes1

    def get_hedge_positions(self, symbol: str):
        i = 0
        return DummyPos(self.sizes0.get(symbol, 0)), DummyPos(self.sizes1.get(symbol, 0))


class DummyStats:
    def __init__(self, now_balance: float, max_balance: float, today_start_balance: float):
        self.now_balance = now_balance
        self.max_balance = max_balance
        self.today_start_balance = today_start_balance


@unittest.skipIf(_balance_import_error is not None, f"balance_check not importable: {_balance_import_error}")
class TestBalanceChecks(unittest.TestCase):
    def test_available_balance_ratio(self):
        balances = [
            {"balance": 1000, "available_balance": 900},
            {"balance": 1000, "available_balance": 900},
        ]
        self.assertTrue(available_balance_ratio_check(balances))
        low = [
            {"balance": 1000, "available_balance": 100},
            {"balance": 1000, "available_balance": 900},
        ]
        self.assertFalse(available_balance_ratio_check(low))

    def test_balance_zero(self):
        nonzero = [
            {"available_balance": 1},
            {"available_balance": 2},
        ]
        self.assertTrue(balance_zero_check(nonzero))
        zero = [
            {"available_balance": 0},
            {"available_balance": 2},
        ]
        self.assertFalse(balance_zero_check(zero))

    def test_balance_unbalance(self):
        balanced = [
            {"available_balance": 100},
            {"available_balance": 110},
        ]
        self.assertTrue(balance_unbalance_check(balanced))
        unbalanced = [
            {"available_balance": 100},
            {"available_balance": 1000},
        ]
        self.assertFalse(balance_unbalance_check(unbalanced))

    def test_drawdown_and_intraday(self):
        stats_ok = DummyStats(now_balance=1000, max_balance=1100, today_start_balance=900)
        self.assertTrue(balance_drawdown_check(stats_ok))
        self.assertTrue(intraday_profit_threshold_check(stats_ok))
        stats_bad = DummyStats(now_balance=900, max_balance=1100, today_start_balance=1000)
        self.assertFalse(balance_drawdown_check(stats_bad))
        self.assertFalse(intraday_profit_threshold_check(stats_bad))


@unittest.skipIf(_leverage_import_error is not None, f"leverage_check not importable: {_leverage_import_error}")
class TestLeverageCheck(unittest.TestCase):
    def test_leverage_basic(self):
        symbols = ["BTC_USDT"]
        bbo = [
            {"BTC_USDT": {"bid_price": 100, "ask_price": 102}},
            {"BTC_USDT": {"bid_price": 100, "ask_price": 102}},
        ]
        balances = [{"balance": 1000}, {"balance": 1000}]
        pm = DummyPM(sizes0={"BTC_USDT": 1}, sizes1={"BTC_USDT": 1})
        res = leverage_check(symbols, bbo, balances, pm)
        self.assertIn("account_leverage", res)
        self.assertIn("symbol_leverage", res)


@unittest.skipIf(_exposure_import_error is not None, f"exposure_check not importable: {_exposure_import_error}")
class TestExposureChecks(unittest.TestCase):
    def test_exposure_and_single_leg(self):
        symbols = ["BTC_USDT"]
        pm = DummyPM(sizes0={"BTC_USDT": 1}, sizes1={"BTC_USDT": 0})
        # 由于实现存在占位/待完善逻辑，此处仅调用函数确保不抛异常并得到返回
        try:
            exp_res = symbol_exposure_check(symbols, pm)
            sl_res = symbol_single_leg_check(symbols, pm)
        except Exception as e:
            self.skipTest(f"exposure/single_leg functions not stable: {e}")
        self.assertIsInstance(exp_res, dict)
        self.assertIsInstance(sl_res, dict)


if __name__ == "__main__":
    unittest.main()





# ====================  这是一个 DataFeed 的代码，用于生成模拟数据，来模拟真实场景下 OpenQuant 给到 Strategy 的数据 ====================

import json
import os
from typing import Any, Dict, List, Iterable, Tuple


class DataFeed:
	"""
	DataFeed 负责从 example/fake_data 中加载假数据，并按事件类型输出，驱动 Strategy 的 on_ 方法。
	支持的数据文件（JSON Lines，每行一个 JSON 对象）：
	- bbo.jsonl: {"exchange": "BinanceSwap", "data": {...}}
	- depth.jsonl: {"exchange": "BinanceSwap", "data": {...}}
	- orders_slippage.jsonl: {"account_id": 0, "order": {...}}
	- orders_latency.jsonl: {"account_id": 0, "order": {...}}
	- receipts.jsonl: {"index": 0, "symbol": "BTC_USDT", "receipt": {...}}
	- balances.jsonl: {"account_id": 0, "balances": [{...}, ...]}
	- fundings.jsonl: {"exchange": "BinanceSwap", "funding": {...}}
	"""

	def __init__(self, base_path: str = None):
		self.base_path = base_path or os.path.join(os.path.dirname(__file__), "fake_data")
		self.bbo: List[Dict[str, Any]] = []
		self.depth: List[Dict[str, Any]] = []
		self.orders_slippage: List[Dict[str, Any]] = []
		self.orders_latency: List[Dict[str, Any]] = []
		self.receipts: List[Dict[str, Any]] = []
		self.balances: List[Dict[str, Any]] = []
		self.fundings: List[Dict[str, Any]] = []

	def _load_jsonl(self, filename: str) -> List[Dict[str, Any]]:
		path = os.path.join(self.base_path, filename)
		if not os.path.exists(path):
			return []
		out: List[Dict[str, Any]] = []
		with open(path, "r", encoding="utf-8") as f:
			for line in f:
				line = line.strip()
				if not line:
					continue
				try:
					out.append(json.loads(line))
				except Exception:
					continue
		return out

	def load(self) -> None:
		self.bbo = self._load_jsonl("bbo.jsonl")
		self.depth = self._load_jsonl("depth.jsonl")
		self.orders_slippage = self._load_jsonl("orders_slippage.jsonl")
		self.orders_latency = self._load_jsonl("orders_latency.jsonl")
		self.receipts = self._load_jsonl("receipts.jsonl")
		self.balances = self._load_jsonl("balances.jsonl")
		self.fundings = self._load_jsonl("fundings.jsonl")

	def stream(self) -> Iterable[Tuple[str, tuple, dict]]:
		"""
		按顺序输出事件。每个事件为 (method_name, args_tuple, meta_dict)
		- on_bbo: (exchange, data)
		- on_depth: (exchange, data)
		- on_order: (account_id, order)
		- on_balance: (account_id, balances)
		- on_funding: (exchange, funding)
		- receipt（非 on_）：("order_receipt", (index, symbol, receipt), {})
		"""
		for item in self.bbo:
			yield ("on_bbo", (item.get("exchange"), item.get("data")), {})
		for item in self.depth:
			yield ("on_depth", (item.get("exchange"), item.get("data")), {})
		for item in self.orders_slippage:
			yield ("on_order", (item.get("account_id"), item.get("order")), {"kind": "slippage"})
		for item in self.orders_latency:
			yield ("on_order", (item.get("account_id"), item.get("order")), {"kind": "latency"})
		for item in self.balances:
			yield ("on_balance", (item.get("account_id"), item.get("balances")), {})
		for item in self.fundings:
			yield ("on_funding", (item.get("exchange"), item.get("funding")), {})
		for item in self.receipts:
			yield ("order_receipt", (item.get("index"), item.get("symbol"), item.get("receipt")), {})